'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var backendCommon = require('@backstage/backend-common');
var express = require('express');
var Router = require('express-promise-router');
var utilArnParser = require('@aws-sdk/util-arn-parser');
var awsCorePluginForBackstageNode = require('@aws/aws-core-plugin-for-backstage-node');
var awsCorePluginForBackstageCommon = require('@aws/aws-core-plugin-for-backstage-common');
var catalogModel = require('@backstage/catalog-model');
var integrationAwsNode = require('@backstage/integration-aws-node');
var clientCodepipeline = require('@aws-sdk/client-codepipeline');
var awsCodepipelinePluginForBackstageCommon = require('@aws/aws-codepipeline-plugin-for-backstage-common');
var backendPluginApi = require('@backstage/backend-plugin-api');
var alpha = require('@backstage/plugin-catalog-node/alpha');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

var express__default = /*#__PURE__*/_interopDefaultCompat(express);
var Router__default = /*#__PURE__*/_interopDefaultCompat(Router);

const DEFAULT_EXECUTIONS_LIMIT = 100;
class DefaultAwsCodePipelineService {
  constructor(logger, auth, catalogApi, resourceLocator, credsManager) {
    this.logger = logger;
    this.auth = auth;
    this.catalogApi = catalogApi;
    this.resourceLocator = resourceLocator;
    this.credsManager = credsManager;
  }
  static async fromConfig(config, options) {
    const credsManager = integrationAwsNode.DefaultAwsCredentialsManager.fromConfig(config);
    const { auth } = backendCommon.createLegacyAuthAdapters(options);
    const resourceLocator = options?.resourceLocator ?? await awsCorePluginForBackstageNode.AwsResourceLocatorFactory.fromConfig(config, options.logger);
    return new DefaultAwsCodePipelineService(
      options.logger,
      auth,
      options.catalogApi,
      resourceLocator,
      credsManager
    );
  }
  async getPipelineExecutionsByEntity(options) {
    this.logger?.debug(`Fetch CodePipeline state for ${options.entityRef}`);
    const arns = await this.getPipelineArnsForEntity(options);
    const pipelineExecutions = await Promise.all(
      arns.map(async (arn) => {
        const { region, resource } = utilArnParser.parse(arn);
        const pipelineName = resource;
        const client = await this.getClient(region, arn);
        const paginatorConfig = {
          client,
          pageSize: 25
        };
        const commandParams = {
          pipelineName
        };
        const paginator = clientCodepipeline.paginateListPipelineExecutions(
          paginatorConfig,
          commandParams
        );
        const pipelineExecutionSummaries = [];
        for await (const page of paginator) {
          const executions = page.pipelineExecutionSummaries || [];
          if (pipelineExecutionSummaries.length + executions.length > DEFAULT_EXECUTIONS_LIMIT) {
            pipelineExecutionSummaries.push(
              ...executions.slice(
                0,
                DEFAULT_EXECUTIONS_LIMIT - pipelineExecutionSummaries.length
              )
            );
          } else {
            pipelineExecutionSummaries.push(...executions);
          }
          if (pipelineExecutionSummaries.length === DEFAULT_EXECUTIONS_LIMIT) {
            break;
          }
        }
        return {
          pipelineName,
          pipelineRegion: region,
          pipelineArn: arn,
          pipelineExecutions: pipelineExecutionSummaries
        };
      })
    );
    return {
      pipelineExecutions
    };
  }
  async getPipelineStateByEntity(options) {
    this.logger?.debug(`Fetch CodePipeline state for ${options.entityRef}`);
    const arns = await this.getPipelineArnsForEntity(options);
    const pipelines = await Promise.all(
      arns.map(async (arn) => {
        const { region, resource } = utilArnParser.parse(arn);
        const pipelineName = resource;
        const client = await this.getClient(region, arn);
        const pipelineState = await client.send(
          new clientCodepipeline.GetPipelineStateCommand({
            name: pipelineName
          })
        );
        return {
          pipelineName,
          pipelineRegion: region,
          pipelineArn: arn,
          pipelineState
        };
      })
    );
    return {
      pipelines
    };
  }
  async getPipelineArnsForEntity(options) {
    const entity = await this.catalogApi.getEntityByRef(
      options.entityRef,
      options.credentials && await this.auth.getPluginRequestToken({
        onBehalfOf: options.credentials,
        targetPluginId: "catalog"
      })
    );
    if (!entity) {
      throw new Error(
        `Couldn't find entity with name: ${catalogModel.stringifyEntityRef(
          options.entityRef
        )}`
      );
    }
    const annotation = awsCorePluginForBackstageCommon.getOneOfEntityAnnotations(entity, [
      awsCodepipelinePluginForBackstageCommon.AWS_CODEPIPELINE_ARN_ANNOTATION,
      awsCodepipelinePluginForBackstageCommon.AWS_CODEPIPELINE_TAGS_ANNOTATION,
      awsCodepipelinePluginForBackstageCommon.AWS_CODEPIPELINE_ARN_ANNOTATION_LEGACY
    ]);
    if (!annotation) {
      throw new Error("Annotation not found on entity");
    }
    let arns;
    if (annotation.name === awsCodepipelinePluginForBackstageCommon.AWS_CODEPIPELINE_TAGS_ANNOTATION) {
      arns = await this.resourceLocator.getResourceArns({
        resourceType: "AWS::CodePipeline::Pipeline",
        tagString: annotation.value
      });
    } else {
      arns = [annotation.value];
    }
    return Promise.resolve(arns);
  }
  async getClient(region, arn) {
    const credentialProvider = await this.credsManager.getCredentialProvider({
      arn
    });
    return new clientCodepipeline.CodePipelineClient({
      region,
      customUserAgent: awsCorePluginForBackstageCommon.AWS_SDK_CUSTOM_USER_AGENT,
      credentialDefaultProvider: () => credentialProvider.sdkCredentialProvider
    });
  }
}
const awsCodePipelineServiceRef = backendPluginApi.createServiceRef({
  id: "aws-codepipeline.api",
  defaultFactory: async (service) => backendPluginApi.createServiceFactory({
    service,
    deps: {
      logger: backendPluginApi.coreServices.logger,
      config: backendPluginApi.coreServices.rootConfig,
      catalogApi: alpha.catalogServiceRef,
      auth: backendPluginApi.coreServices.auth,
      discovery: backendPluginApi.coreServices.discovery,
      httpAuth: backendPluginApi.coreServices.httpAuth
    },
    async factory({
      logger,
      config,
      catalogApi,
      auth,
      httpAuth,
      discovery
    }) {
      const impl = await DefaultAwsCodePipelineService.fromConfig(config, {
        catalogApi,
        auth,
        httpAuth,
        discovery,
        logger
      });
      return impl;
    }
  })
});

async function createRouter(options) {
  const { logger, awsCodePipelineApi } = options;
  const router = Router__default.default();
  router.use(express__default.default.json());
  const { httpAuth } = backendCommon.createLegacyAuthAdapters(options);
  router.get(
    "/v1/entity/:namespace/:kind/:name/executions",
    async (request, response) => {
      const { namespace, kind, name } = request.params;
      const services = await awsCodePipelineApi.getPipelineExecutionsByEntity({
        entityRef: {
          kind,
          namespace,
          name
        },
        credentials: await httpAuth.credentials(request)
      });
      response.status(200).json(services);
    }
  );
  router.get(
    "/v1/entity/:namespace/:kind/:name/state",
    async (request, response) => {
      const { namespace, kind, name } = request.params;
      const services = await awsCodePipelineApi.getPipelineStateByEntity({
        entityRef: {
          kind,
          namespace,
          name
        },
        credentials: await httpAuth.credentials(request)
      });
      response.status(200).json(services);
    }
  );
  router.get("/health", (_, response) => {
    logger.info("PONG!");
    response.json({ status: "ok" });
  });
  router.use(backendCommon.errorHandler());
  return router;
}

const awsCodePiplinePlugin = backendPluginApi.createBackendPlugin({
  pluginId: "aws-codepipeline",
  register(env) {
    env.registerInit({
      deps: {
        logger: backendPluginApi.coreServices.logger,
        httpRouter: backendPluginApi.coreServices.httpRouter,
        config: backendPluginApi.coreServices.rootConfig,
        catalogApi: alpha.catalogServiceRef,
        auth: backendPluginApi.coreServices.auth,
        discovery: backendPluginApi.coreServices.discovery,
        httpAuth: backendPluginApi.coreServices.httpAuth,
        awsCodePipelineApi: awsCodePipelineServiceRef
      },
      async init({
        logger,
        httpRouter,
        auth,
        httpAuth,
        discovery,
        awsCodePipelineApi
      }) {
        httpRouter.use(
          await createRouter({
            logger,
            awsCodePipelineApi,
            discovery,
            auth,
            httpAuth
          })
        );
        httpRouter.addAuthPolicy({
          path: "/health",
          allow: "unauthenticated"
        });
      }
    });
  }
});

exports.DefaultAwsCodePipelineService = DefaultAwsCodePipelineService;
exports.awsCodePipelineServiceRef = awsCodePipelineServiceRef;
exports.createRouter = createRouter;
exports.default = awsCodePiplinePlugin;
//# sourceMappingURL=index.cjs.js.map
